/* OggDropXPd - Filename: AUDIO.H
 *
 *   Copyright (c) 2002 - 2005 John Edwards <john.edwards33@ntlworld.com>
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Portions Copyright 2000-2002, Michael Smith <msmith@labyrinth.net.au>
 *
 * AIFF/AIFC support from OggSquish, (c) 1994-1996 Monty <xiphmont@xiph.org>
 */


#ifndef AUDIO_H_INCLUDED
#define AUDIO_H_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

#include "encode.h"
#include <stdio.h>

#define READSIZE 1152

#define MAXWAVESIZE     4294967040LU

#ifndef	WAVE_FORMAT_PCM
#	define	WAVE_FORMAT_PCM                0x0001
#endif
#ifndef	WAVE_FORMAT_IEEE_FLOAT
#	define	WAVE_FORMAT_IEEE_FLOAT         0x0003
#endif
#ifndef	WAVE_FORMAT_EXTENSIBLE
#	define	WAVE_FORMAT_EXTENSIBLE         0xfffe
#endif

typedef struct
{
	int  (*id_func)(unsigned char *buf, int len); /* Returns true if can load file */
	int  id_data_len; /* Amount of data needed to id whether this can load the file */
	int  (*open_func)(FILE *in, mp3data_struct *mp3input_data, le_enc_opt *opt, unsigned char *buf, int buflen);
	void (*close_func)(void *);
	char *format;
	char *description;
} InputFormat;


typedef struct {
	unsigned short format;
	unsigned short channels;
	unsigned int   samplerate;
	unsigned int   bytespersec;
	unsigned short align;
	unsigned short samplesize;
} wav_fmt;

typedef struct {
	short channels;
	short samplesize;
	long  totalsamples;
	long  samplesread;
	FILE  *f;
	short bigendian;
} wavfile;

typedef struct {
	short channels;
	int   totalframes;
	short samplesize;
	int   rate;
	int   offset;
	int   blocksize;
} aiff_fmt;

typedef wavfile aifffile; /* They're the same */

InputFormat *open_audio_file(FILE *in, mp3data_struct *mp3input_data, le_enc_opt *opt);

int wav_open(FILE *in, mp3data_struct *mp3input_data, le_enc_opt *opt, unsigned char *buf, int buflen);
int aiff_open(FILE *in, mp3data_struct *mp3input_data, le_enc_opt *opt, unsigned char *buf, int buflen);
int wav_id(unsigned char *buf, int len);
int aiff_id(unsigned char *buf, int len);
void wav_close(void *);

void setup_scaler(le_enc_opt *opt, float scale);
void clear_scaler(le_enc_opt *opt);

long wav_read(void *, float **buffer, int samples);
long wav_ieee_read(void *, float **buffer, int samples);

#ifdef __cplusplus
}
#endif
#endif /* AUDIO_H_INCLUDED */
