/*
 *      Stripped down decoder frontend for lamedropXPd
 *
 *      Copyright (c) 1999 Mark Taylor
 *                    2000 Takehiro TOMIANGA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: main.c,v 1.73 2001/10/29 22:00:15 markt Exp $ */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <assert.h>
#include <stdio.h>

#ifdef STDC_HEADERS
# include <stdlib.h>
# include <string.h>
#else
# ifndef HAVE_STRCHR
#  define strchr index
#  define strrchr rindex
# endif
char   *strchr(), *strrchr();
#endif

#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#endif

#if defined(_WIN32)
# include <windows.h>
#endif


#include "lame.h"

#include "get_audio.h"
#include "portableio.h"
#include "encode.h"


#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

int enc_delay=-1;
int disable_wav_header=0;
int enc_padding=-1;

/* the simple lame decoder */
/* After calling lame_init(), lame_init_params() and
 * init_infile(), call this routine to read the input MP3 file
 * and output .wav data to the specified file pointer*/
/* lame_decoder will ignore the first 528 samples, since these samples
 * represent the mpglib delay (and are all 0).  skip = number of additional
 * samples to skip, to (for example) compensate for the encoder delay */

int
lame_decoder(lame_global_flags * gfp, FILE * outf, int skip_start, char *inPath,
             char *outPath, le_enc_opt *opt, int *enc_delay, int *enc_padding)
{
	short int Buffer[2][1152];
	int     iread;
	int     skip_end=0;
	double  wavsize;
	int     i;
	void    (*WriteFunction) (FILE * fp, char *p, int n);
	int tmp_num_channels = lame_get_num_channels( gfp );

    if (!(tmp_num_channels >= 1 && tmp_num_channels <= 2)) {
        error_handler("Internal error.  Aborting.");
        exit(-1);
    }

	switch (input_format) {
		case sf_mp3:
			if (skip_start == 0) {
				if (*enc_delay > -1 || *enc_padding > -1) {
					if (*enc_delay > -1) 
						skip_start = *enc_delay + 528 + 1;
					if (*enc_padding > -1) 
						skip_end = *enc_padding - (528 + 1);
				}
                                else 
					skip_start=lame_get_encoder_delay(gfp) + 528 + 1;
			}
                        else {
				// user specified a value of skip. just add for decoder
				skip_start += 528 + 1; /* mp3 decoder has a 528 sample delay, plus user supplied "skip" */
			}
			break;
		case sf_mp2:
			skip_start += 240 + 1;
			break;
		case sf_mp1:
			skip_start += 240 + 1;
			break;
		case sf_ogg:
			skip_start = 0;
			break;
		default:
			mp3input_data.nsamp = lame_get_num_samples( gfp );
			mp3input_data.framesize = 1152;
			skip_start = 0;       /* other formats have no delay *//* is += 0 not better ??? */
			assert(0);
			break;
	}

	if (0 == disable_wav_header)
		WriteWaveHeader(outf, 0x7FFFFFFF, lame_get_in_samplerate(gfp), tmp_num_channels, 16);
		/* unknown size, so write maximum 32 bit signed value */

	wavsize = -(skip_start + skip_end);
	WriteFunction = swapbytes ? WriteBytesSwapped : WriteBytes;
	mp3input_data.totalframes = mp3input_data.nsamp / mp3input_data.framesize;

	assert(tmp_num_channels >= 1 && tmp_num_channels <= 2);

	do {
		iread = get_audio16(gfp, Buffer, opt); /* read in 'iread' samples */
            if (iread >= 0) {
		mp3input_data.framenum = (int)(wavsize / mp3input_data.framesize);
		wavsize += iread;

		opt->progress_update(mp3input_data.totalframes, mp3input_data.framenum);

		skip_start -= (i = skip_start < iread ? skip_start : iread); /* 'i' samples are to skip in this frame */

		if (skip_end > 1152 && mp3input_data.framenum + 2 > mp3input_data.totalframes) {
			iread -= (skip_end - 1152);
			skip_end = 1152;
		}
		else if (mp3input_data.framenum == mp3input_data.totalframes && iread != 0)
			iread -= skip_end;

		for (; i < iread; i++) {
			if (disable_wav_header) {
				WriteFunction(outf, (char *) &Buffer[0][i], sizeof(short));
				if (tmp_num_channels == 2)
					WriteFunction(outf, (char *) &Buffer[1][i], sizeof(short));
			}
			else {
				Write16BitsLowHigh(outf, Buffer[0][i]);
				if (tmp_num_channels == 2)
					Write16BitsLowHigh(outf, Buffer[1][i]);
			}
		}
            }
	} while (iread);

	i = (16 / 8) * tmp_num_channels;
	assert(i > 0);
	if (wavsize <= 0)
		wavsize = 0;
	else if (wavsize > 0xFFFFFFD0 / i)
		wavsize = 0xFFFFFFD0;
	else
		wavsize *= i;

	if (0 == disable_wav_header)
		if (!fseek(outf, 0l, SEEK_SET)) /* if outf is seekable, rewind and adjust length */
			WriteWaveHeader(outf, (int)wavsize, lame_get_in_samplerate( gfp ), tmp_num_channels, 16);

	return 0;
}
/*
 * end of decode.c 
 */
